@@ -1,22 +1,27 @@
-from __future__ import absolute_import, print_function
+from __future__ import generator_stop
 
 import os
 import stat
 import re
 import errno
 
+from . import git_handler
+from . import gitrepo
+
 from mercurial import (
     dirstate,
     error,
+    exthelper,
     match as matchmod,
     pathutil,
     pycompat,
     util,
 )
 
-from . import compat
 from mercurial.i18n import _
 
+eh = exthelper.exthelper()
+
 
 def gignorepats(orig, lines, root=None):
     '''parse lines (iterable) of .gitignore text, returning a tuple of
@@ -52,12 +57,12 @@ def gignorepats(orig, lines, root=None):
             rootsuffixes = [b'', b'**/']
         for rootsuffix in rootsuffixes:
             pat = syntax + rootprefix + rootsuffix + line
-            for s, rels in compat.iteritems(syntaxes):
+            for s, rels in syntaxes.items():
                 if line.startswith(rels):
                     pat = line
                     break
                 elif line.startswith(s + b':'):
-                    pat = rels + line[len(s) + 1:]
+                    pat = rels + line[len(s) + 1 :]
                     break
             patterns.append(pat)
 
@@ -94,12 +99,8 @@ class gitdirstate(dirstate.dirstate):
                 files.append(util.expandpath(path))
         patterns = []
         # Only use .gitignore if there's no .hgignore
-        try:
-            fp = open(files[0], 'rb')
-            fp.close()
-        except:
-            fns = self._finddotgitignores()
-            for fn in fns:
+        if not os.access(files[0], os.R_OK):
+            for fn in self._finddotgitignores():
                 d = os.path.dirname(fn)
                 fn = self.pathto(fn)
                 if not os.path.exists(fn):
@@ -115,20 +116,25 @@ class gitdirstate(dirstate.dirstate):
         """A copy of dirstate.walk. This is called from the new _ignore method,
         which is called by dirstate.walk, which would cause infinite recursion,
         except _finddotgitignores calls the superclass _ignore directly."""
-        match = matchmod.match(self._root, self.getcwd(),
-                               [b'relglob:.gitignore'])
+        match = matchmod.match(
+            self._root, self.getcwd(), [b'relglob:.gitignore']
+        )
         # TODO: need subrepos?
         subrepos = []
         unknown = True
         ignored = False
 
         def fwarn(f, msg):
-            self._ui.warn(b'%s: %s\n' % (
-                self.pathto(f), pycompat.sysbytes(msg),
-            ))
+            self._ui.warn(
+                b'%s: %s\n'
+                % (
+                    self.pathto(f),
+                    pycompat.sysbytes(msg),
+                )
+            )
             return False
 
-        ignore = super(gitdirstate, self)._ignore
+        ignore = super()._ignore
         dirignore = self._dirignore
         if ignored:
             ignore = util.never
@@ -151,7 +157,7 @@ class gitdirstate(dirstate.dirstate):
         exact = skipstep3 = False
         if matchfn == match.exact:  # match.exact
             exact = True
-            dirignore = util.always                  # skip step 2
+            dirignore = util.always  # skip step 2
         elif match.files() and not match.anypats():  # match.match, no patterns
             skipstep3 = True
 
@@ -172,10 +178,6 @@ class gitdirstate(dirstate.dirstate):
         while work:
             nd = work.pop()
             skip = None
-            if nd == b'.':
-                # <= hg-5.0 use "." for the root directory
-                # (see Mercurial-core27d6956d386b for details)
-                nd = b''
             if nd != b'':
                 skip = b'.hg'
             try:
@@ -253,4 +255,13 @@ class gitdirstate(dirstate.dirstate):
             self._ui.debug(b'suppressing rust status to intercept gitignores\n')
             raise dirstate.rustmod.FallbackError
         else:
-            return super(gitdirstate, self)._rust_status(*args, **kwargs)
+            return super()._rust_status(*args, **kwargs)
+
+
+@eh.reposetup
+def reposetup(ui, repo):
+    if isinstance(repo, gitrepo.gitrepo):
+        return
+
+    if git_handler.has_gitrepo(repo):
+        dirstate.dirstate = gitdirstate
