@@ -4,22 +4,32 @@
 #
 # incomplete, implemented on demand
 
-from __future__ import absolute_import, print_function
+from __future__ import generator_stop
 
 from mercurial import (
     ancestor,
     changelog,
     context,
+    exthelper,
     manifest,
     match as matchmod,
     namespaces,
     node,
     util,
 )
+from mercurial.utils import stringutil
 from mercurial.node import bin, hex, nullid
 
+from dulwich.refs import (
+    LOCAL_BRANCH_PREFIX,
+    LOCAL_TAG_PREFIX,
+)
+
 from . import compat
 
+eh = exthelper.exthelper()
+
+
 def _maybehex(n):
     if len(n) == 20:
         return hex(n)
@@ -35,11 +45,7 @@ class overlaymanifest(object):
 
     def withflags(self):
         self.load()
-        return {
-            path
-            for path, flag in compat.iteritems(self._flags)
-            if flag != b''
-        }
+        return {path for path, flag in self._flags.items() if flag != b''}
 
     def copy(self):
         return overlaymanifest(self.repo, self.tree.id)
@@ -67,7 +73,7 @@ class overlaymanifest(object):
                 return b''
 
         def addtree(tree, dirname):
-            for entry in compat.iteritems(tree):
+            for entry in tree.items():
                 if entry.mode & 0o40000:
                     # expand directory
                     subtree = self.repo.handler.git.get_object(entry.sha)
@@ -92,7 +98,7 @@ class overlaymanifest(object):
 
     def iteritems(self):
         self.load()
-        return compat.iteritems(self._map)
+        return self._map.items()
 
     def __iter__(self):
         self.load()
@@ -149,6 +155,7 @@ class overlaymanifest(object):
         del self._map[path]
 
 
+@eh.wrapfunction(manifest.manifestdict, b'diff')
 def wrapmanifestdictdiff(orig, self, m2, match=None, clean=False):
     '''avoid calling into lazymanifest code if m2 is an overlaymanifest'''
     # Older mercurial clients used diff(m2, clean=False). If a caller failed
@@ -157,9 +164,7 @@ def wrapmanifestdictdiff(orig, self, m2, match=None, clean=False):
         clean = match
         match = None
 
-    kwargs = {
-        'clean': clean
-    }
+    kwargs = {'clean': clean}
     # Older versions of mercurial don't support the match arg, so only add it
     # if it exists.
     if match is not None:
@@ -205,7 +210,7 @@ class overlayfilectx(object):
         return blob.data
 
     def isbinary(self):
-        return compat.binary(self.data())
+        return stringutil.binary(self.data())
 
 
 class overlaychangectx(context.changectx):
@@ -214,7 +219,7 @@ class overlaychangectx(context.changectx):
         self._hgrepo = repo
         if isinstance(sha, bytes):
             pass
-        elif isinstance(sha, compat.unicode):
+        elif isinstance(sha, str):
             sha = sha.encode('ascii')
         else:
             sha = sha.hex()
@@ -229,7 +234,7 @@ class overlaychangectx(context.changectx):
         # So, if the attribute wasn't found, fallback to _hgrepo
         if name == '_repo':
             return self._hgrepo
-        return super(overlaychangectx, self).__getattr__(name)
+        return super().__getattr__(name)
 
     def repo(self):
         return self._hgrepo
@@ -291,20 +296,26 @@ class overlaychangectx(context.changectx):
         mf = self.manifest()
         return mf.flags(path)
 
-    def __nonzero__(self):
+    def __bool__(self):
         return True
-    __bool__ = __nonzero__
 
     def phase(self):
         try:
             from mercurial import phases
+
             return phases.draft
         except (AttributeError, ImportError):
             return 1
 
     def totuple(self):
-        return (self.commit.tree, self.user(), self.date(), self.files(),
-                self.description(), self.extra())
+        return (
+            self.commit.tree,
+            self.user(),
+            self.date(),
+            self.files(),
+            self.description(),
+            self.extra(),
+        )
 
 
 class overlayrevlog(object):
@@ -456,8 +467,9 @@ class overlayrepo(object):
         self.githandler = overlaygithandler(self)
 
     def _constructmanifest(self):
-        return overlaymanifestrevlog(self,
-                                     self.handler.repo._constructmanifest())
+        return overlaymanifestrevlog(
+            self, self.handler.repo._constructmanifest()
+        )
 
     def __getitem__(self, n):
         if isinstance(n, int):
@@ -521,13 +533,12 @@ class overlayrepo(object):
         self.refmap = {}
         self.tagmap = {}
         for ref in refs:
-            if ref.startswith(b'refs/heads/'):
-                refname = ref[11:]
+            if ref.startswith(LOCAL_BRANCH_PREFIX):
+                refname = ref[len(LOCAL_BRANCH_PREFIX) :]
                 self.refmap.setdefault(bin(refs[ref]), []).append(refname)
-            elif ref.startswith(b'refs/tags/'):
-                tagname = ref[10:]
+            elif ref.startswith(LOCAL_TAG_PREFIX):
+                tagname = ref[len(LOCAL_TAG_PREFIX) :]
                 self.tagmap.setdefault(bin(refs[ref]), []).append(tagname)
 
     def narrowmatch(self, *args, **kwargs):
         return self.handler.repo.narrowmatch(*args, **kwargs)
-
